/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// logfile.cxx                                     1998 SOS Software Gmbh
// Joacim Zschimmer
/*
    Schreibt ins Log, wenn es geffnet ist (s. soslog.cxx).
*/

#include "precomp.h"

#include "../kram/sos.h"
#include "../file/absfile.h"
#include "../kram/sosopt.h"
#include "../kram/log.h"


namespace sos {
using namespace std;


//------------------------------------------------------------------------------Log_file

struct Log_file : Abs_file 
{
    void                            open                ( const char*, Open_mode, const File_spec& );

  protected:
    void                            put_record          ( const Const_area& );

    Open_mode                      _open_mode;
};

//--------------------------------------------------------------------------Log_file_type

struct Log_file_type : Abs_file_type
{
    virtual const char*         name                    () const { return "log"; }
    virtual Sos_ptr<Abs_file>   create_base_file        () const
    {
        Sos_ptr<Log_file> f = SOS_NEW( Log_file() );
        return +f;
    }
};

const Log_file_type      _log_file_type;
const Abs_file_type&      log_file_type = _log_file_type;

// -------------------------------------------------------------------------Log_file::open

void Log_file::open( const char* param, Open_mode open_mode, const File_spec& file_spec )
{
    Sos_string filename;
    Bool       start = false;
    Bool       stop  = false;

    if( ( open_mode & inout ) == 0 )  open_mode = File_base::Open_mode( open_mode | out );
    if( ( open_mode & inout ) != out )  throw_xc( "SOS-1388" );
    _open_mode = open_mode;

    for( Sos_option_iterator opt ( param ); !opt.end(); opt.next() )
    {
        if( opt.flag( "start" ) )           start = opt.set();
        else
        if( opt.flag( "stop" ) )            stop = opt.set();
        else
        if( opt.param() || opt.pipe() )     filename = opt.rest();
        else throw_sos_option_error( opt );
    }

    // berprfung der Argumente
  //int HIER_MUESSEN_NOCH_RICHTIGE_MELDUNGEN_REIN;
    if ( start && stop )                      throw_xc( "Log_file-start/stop" );
    if ( start && filename == empty_string )  throw_xc( "empty_log" );
                    
    if ( start ) {
        if ( log_ptr ) log_stop();
        log_start( c_str(filename) );
    } else if ( stop ) {
        log_stop();
    }
}

// ----------------------------------------------------------------------Log_file::put_record

void Log_file::put_record( const Const_area& record )
{
    if( log_ptr ) {
        *log_ptr << record;
        if( !( _open_mode & binary ) )  *log_ptr << '\n';
        *log_ptr << flush;
    }
}

} //namespace sos
