/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// log.h                                                (c) SOS GmbH Berlin
//                                                          Joacim Zschimmer

#ifndef __LOG_H
#define __LOG_H

#include "../kram/sosstat.h"


namespace sos
{


extern void                     sos_log                     ( const char* );
extern void*                    log_create                  ( const char* filename = 0 );
extern void                     log_start                   ( const char* filename = 0 );
extern void                     log_stop                    ();
inline string                   log_filename                ()                              { return sos_static_ptr()->_log_filename; }
inline string                   current_log_categories      ()                              { return sos_static_ptr()->_log_categories; }


#if defined __SOSSTRNG_H
inline void log_start( const Sos_string& filename )
{
    log_start( c_str( filename ) );
}
#endif

#define _LOG( text )       do { ::sos::Log_ptr _l_; if( _l_ )  { *_l_ << text; _l_->flush(); } } while(0)

#if defined DONT_LOG
#   define VOID_LOG( text )  ((void)0)
#   define LOG( text )       do {} while(0)
#   define LOGI( text )      LOG( text ); ;
#else
#   define VOID_LOG( text )  ( log_ptr? (void)( *log_ptr << text ) : (void)0 )
#   define LOG( text )       _LOG( text )
#   define LOG_INDENT_       ::sos::Log_indent __log_indent__##__LINE__;
#   define LOGI( text )      LOG( text );  LOG_INDENT_
#endif

#define LOG_ERR( text )   _LOG( text )   // Fehler

//---------------------------------------------------------------------------........log_categories

using zschimmer::log_category_is_set;
using zschimmer::set_log_category;
using zschimmer::set_log_category_default;
using zschimmer::set_log_category_implicit;
using zschimmer::set_log_category_explicit;
using zschimmer::log_categories_as_string;

#define LOG2( CATEGORY, TEXT )   if( log_category_is_set( CATEGORY ) )  { LOG( TEXT ); }  else;  
#define LOGI2( CATEGORY, TEXT )  if( log_category_is_set( CATEGORY ) )  { LOG( TEXT ); }  LOG_INDENT_

//-------------------------------------------------------------------------------------------------

void                            log_indent                  ( int direction );      // +1 oder -1

//------------------------------------------------------------------------------------------Log_ptr

struct Log_ptr
{
                                Log_ptr                     ()                                      : _s( sos_static_ptr()->_log_ptr ) { if( _s ) sos_static_ptr()->_log_lock.enter(); }
                               ~Log_ptr                     ()                                      { if( _s ) sos_static_ptr()->_log_lock.leave(); }

                                operator ostream*           ()                                      { return _s; }
    ostream&                    operator *                  ()                                      { return *_s; }
    ostream*                    operator ->                 ()                                      { return _s; }

    static int                  log_write                   ( const char*, int length );

    ostream*                   _s;
};

#define log_ptr ( Log_ptr()._s )
// Damit sollte LOG() und *log_ptr << ... << ...  durch die Semaphore serialisiert sein.

//#define log_ptr ( sos::sos_static_ptr()->_log_ptr )

//---------------------------------------------------------------------------------------Log_indent

struct Log_indent
{
                                Log_indent                  ( Sos_static* s = sos::sos_static_ptr() );
                               ~Log_indent                  ();

  private:

  //static int                 _log_indent_tls;

    Sos_static* _static_ptr;
};


/*
#if defined DONT_LOG

    struct Log_block
    {
        Log_block( const char* ) {}
    };

#else

    struct Log_block
    {
        Log_block( const char* text_ptr )
        :
            _text_ptr( text_ptr ),
            _static_ptr ( sos_static_ptr() )
        {
            LOG( _text_ptr << " begin\n" );
            _static_ptr->_log_indent++;
        }

        ~Log_block()
        {
            _static_ptr->_log_indent--;
            LOG( _text_ptr << " end\n" );
        }

      private:
        Sos_static* _static_ptr;
        const char* _text_ptr;
    };

#endif
*/

} //namespace sos

#endif
